///////////////////////////////////////////////////////////////////////////////
// Projet :  Recherche de solution du Solitaire
// Fichier : main.cpp
// Auteur :  Sylvain Togni
// Date :    08/03/2009
///////////////////////////////////////////////////////////////////////////////
#include "Solveur.h"
#include "Plateau.h"
#include <algorithm>
#include <fstream>
#include <iostream>
#include <ctime>
#include <stdexcept>
#include <locale>

///////////////////////////////////////////////////////////////////////////////
// Fonction principale : lecture du plateau et lancement de la recherche
///////////////////////////////////////////////////////////////////////////////
static void rechercheSolution(char const* nomDeFichier)
{
    // Lecture du fichier et vrification du nombre de cases vides
    std::cout << "Lecture du fichier " << nomDeFichier << "...";
    std::ifstream fichier(nomDeFichier);
    Plateau p;
    fichier >> p;
    if (p.nombreDeCasesVides() != 1)
    {
        throw std::logic_error("nombre de cases vides invalide");
    }
    std::cout << "OK\n\n";
    std::cout << p << "\n\n";

    // Recherche d'une solution
    std::cout << "Recherche d'une solution...";
    Solveur solveur;
    std::clock_t t = std::clock();
    bool ok = solveur.rechercheChemin(p, p.inverse());
    t = std::clock() - t;
    if (ok)
    {
        std::cout << "OK\n\n";
        std::cout << solveur.solution() << "\n\n";
    }
    else
    {
        std::cout << "ERREUR\n\n";
        std::cout << "Ce plateau ne possde pas de solution !\n\n";
    }

    // Affichage des infos
    std::cout << "Nombre de positions explores : "
              << solveur.nombreDePositions() << '\n';
    std::cout << "Mmoire utilise : "
              << solveur.memoireUtilisee()/1024 << " Ko\n";
    std::cout << "Temps de recherche : "
              << static_cast<double>(t)/CLOCKS_PER_SEC << " s\n";
}

///////////////////////////////////////////////////////////////////////////////
// Fonction main : gestion des options et des erreurs
///////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
    try
    {
#ifdef _MSC_VER
        // Pour les accents dans la console
        std::locale::global(std::locale(""));
#endif

        // Options ligne de commande
        if (argc == 2 && std::string(argv[1]) == "-v")
        {
            std::cout << "Copyright (c) 2009 Sylvain Togni\n";
        }
        else if (argc == 2 && std::string(argv[1]) == "-h")
        {
            std::cout << "Usages :\n";
            std::cout << "    Solitaire -h            Affiche l'aide\n";
            std::cout << "    Solitaire -v            Affiche le copyright\n";
            std::cout << "    Solitaire -f [fichier]  Charge un fichier "
                         "plateau et recherche une solution\n";
        }
        else if (argc == 3 && std::string(argv[1]) == "-f")
        {
            rechercheSolution(argv[2]);
        }
        else
        {
            throw std::logic_error("ligne de commande invalide (essayez "
                                   "'-h' pour obtenir de l'aide)");
        }
    }
    catch(std::exception& e)
    {
        // gestion des erreurs
        std::cerr << "ERREUR : " << e.what() << " !\n";
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
